@@ -121,12 +121,19 @@ extern unsigned int _fmode;
 
 #endif  /* __i386__ */
 
+#ifndef MB_CUR_MAX
 _ACRTIMP int             __cdecl ___mb_cur_max_func(void);
+_ACRTIMP int             __cdecl ___mb_cur_max_l_func(_locale_t);
 #define __mb_cur_max             ___mb_cur_max_func()
+#define MB_CUR_MAX               ___mb_cur_max_func()
+#endif /* MB_CUR_MAX */
+
 _ACRTIMP __msvcrt_ulong* __cdecl __doserrno(void);
 #define _doserrno              (*__doserrno())
 _ACRTIMP int*            __cdecl _errno(void);
 #define errno                  (*_errno())
+_ACRTIMP int*            __cdecl __sys_nerr(void);
+#define _sys_nerr              (*__sys_nerr())
 
 /* FIXME: We need functions to access these:
  * int _sys_nerr;
@@ -152,12 +159,12 @@ _ACRTIMP int           __cdecl _atodbl_l(_CRT_DOUBLE*,char*,_locale_t);
 _ACRTIMP int           __cdecl _atoflt(_CRT_FLOAT*,char*);
 _ACRTIMP int           __cdecl _atoflt_l(_CRT_FLOAT*,char*,_locale_t);
 _ACRTIMP __int64       __cdecl _atoi64(const char*);
-_ACRTIMP long double   __cdecl _atold(const char*);
 _ACRTIMP int           __cdecl _atoldbl(_LDOUBLE*,char*);
 _ACRTIMP void          __cdecl _beep(unsigned int,unsigned int);
 _ACRTIMP unsigned short   __cdecl _byteswap_ushort(unsigned short);
 _ACRTIMP __msvcrt_ulong   __cdecl _byteswap_ulong(__msvcrt_ulong);
 _ACRTIMP unsigned __int64 __cdecl _byteswap_uint64(unsigned __int64);
+_ACRTIMP errno_t       __cdecl _dupenv_s(char**,size_t*,const char*);
 _ACRTIMP char*         __cdecl _ecvt(double,int,int*,int*);
 _ACRTIMP char*         __cdecl _fcvt(double,int,int*,int*);
 _ACRTIMP char*         __cdecl _fullpath(char*,const char*,size_t);
@@ -203,7 +210,7 @@ _ACRTIMP DECLSPEC_NORETURN void __cdecl _Exit(int);
 _ACRTIMP DECLSPEC_NORETURN void __cdecl _exit(int);
 _ACRTIMP DECLSPEC_NORETURN void __cdecl abort(void);
 _ACRTIMP int           __cdecl abs(int);
-_ACRTIMP int           __cdecl atexit(void (__cdecl *)(void));
+extern int             __cdecl atexit(void (__cdecl *)(void));
 _ACRTIMP double        __cdecl atof(const char*);
 _ACRTIMP int           __cdecl atoi(const char*);
 _ACRTIMP int           __cdecl _atoi_l(const char*,_locale_t);
@@ -227,6 +234,14 @@ _ACRTIMP void          __cdecl srand(unsigned int);
 _ACRTIMP float         __cdecl strtof(const char*,char**);
 _ACRTIMP float         __cdecl _strtof_l(const char*,char**,_locale_t);
 _ACRTIMP double        __cdecl strtod(const char*,char**);
+_ACRTIMP double        __cdecl _strtod_l(const char*,char**,_locale_t);
+#if defined(__GNUC__) || _MSVCR_VER < 120
+static inline long double strtold(const char *string, char **endptr) { return strtod(string, endptr); }
+static inline long double _strtold_l(const char *string, char **endptr, _locale_t locale) { return _strtod_l(string, endptr, locale); }
+#else
+_ACRTIMP long double   __cdecl strtold(const char*,char**);
+_ACRTIMP long double   __cdecl _strtold_l(const char*,char**,_locale_t);
+#endif
 _ACRTIMP __msvcrt_long __cdecl strtol(const char*,char**,int);
 _ACRTIMP __msvcrt_ulong __cdecl strtoul(const char*,char**,int);
 _ACRTIMP __int64       __cdecl _strtoll_l(const char*,char**,int,_locale_t);
@@ -238,6 +253,8 @@ _ACRTIMP unsigned __int64 __cdecl _strtoui64_l(const char*,char**,int,_locale_t)
 _ACRTIMP int           __cdecl system(const char*);
 _ACRTIMP void*         __cdecl bsearch(const void*,const void*,size_t,size_t,int (__cdecl *)(const void*,const void*));
 _ACRTIMP void          __cdecl qsort(void*,size_t,size_t,int (__cdecl *)(const void*,const void*));
+_ACRTIMP void          __cdecl qsort_s(void*,size_t,size_t,int (__cdecl *)(void*,const void*,const void*),void*);
+_ACRTIMP unsigned int  __cdecl _set_abort_behavior(unsigned int flags, unsigned int mask);
 
 typedef void (__cdecl *_purecall_handler)(void);
 _ACRTIMP _purecall_handler __cdecl _set_purecall_handler(_purecall_handler);
@@ -251,11 +268,6 @@ _ACRTIMP _invalid_parameter_handler __cdecl _set_thread_local_invalid_parameter_
 void __cdecl _invalid_parameter(const wchar_t *expr, const wchar_t *func, const wchar_t *file,
                                 unsigned int line, uintptr_t arg);
 
-#ifdef _UCRT
-_ACRTIMP double __cdecl _strtold_l(const char*,char**,_locale_t);
-static inline long double strtold(const char *string, char **endptr) { return _strtold_l(string, endptr, NULL); }
-#endif /* _UCRT */
-
 #ifdef __cplusplus
 extern "C++" {
 
@@ -265,6 +277,9 @@ inline errno_t getenv_s(size_t *ret, char (&buf)[size], const char *var)
     return getenv_s(ret, buf, size, var);
 }
 
+inline long abs(long const x) throw() { return labs(x); }
+inline long long abs(long long const x) throw() { return llabs(x); }
+
 } /* extern "C++" */
 
 }
